'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _camelCaseToDashed = require('./camel-case-to-dashed');

var _camelCaseToDashed2 = _interopRequireDefault(_camelCaseToDashed);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Schema = function () {
  function Schema(definition) {
    _classCallCheck(this, Schema);

    if (definition === undefined) {
      throw new Error('You have to provide a schema definition as parameter.');
    }
    this.definition = {
      id: definition.id || 'id',
      type: definition.type,
      attributes: definition.attributes || {},
      relationships: definition.relationships || {}
    };
  }

  _createClass(Schema, [{
    key: 'resolveIdField',
    value: function resolveIdField(definition, obj, parentObj) {
      if (typeof definition.id === 'function') {
        return definition.id(obj, parentObj);
      } else {
        return obj[definition.id];
      }
    }
  }, {
    key: 'serializeObj',
    value: function serializeObj(obj) {
      var _this = this;

      var attributes = {};

      Object.keys(this.definition.attributes).forEach(function (targetKey) {
        var source = _this.definition.attributes[targetKey];
        var target = (0, _camelCaseToDashed2.default)(targetKey);

        if (typeof source === 'function') {
          attributes[target] = source(obj);
        } else {
          attributes[target] = obj[source];
        }
      });

      var id = this.resolveIdField(this.definition, obj);

      // relationships
      var relationships = {};
      Object.keys(this.definition.relationships).forEach(function (key) {
        var relationship = _this.definition.relationships[key];
        // TODO: support for definitions in relationships
        // const relationshipDefinition = this.definition.relationships[key];

        var relationshipIsArray = Array.isArray(obj[key]);

        // ensure we have an array of objects in the end or empty object if value was undefined before
        var relationshipData = Array.isArray(obj[key] || []) ? obj[key] || [] : [obj[key]];

        var data = relationshipData.map(function (relationshipObject) {
          return _this.serializeRelationship(relationship, relationshipObject, obj);
        });
        // try to call serialize relationship directly and see if someone uses the parentObject to build the relationship
        if (Array.isArray(data) && data.length === 0) {
          var directData = _this.serializeRelationship(relationship, relationshipData, obj);
          if (directData.id) {
            data = directData;
            relationshipIsArray = false; // FIXME: might be wrong, but we need a different api anyhow
          }
        }

        if (!relationshipIsArray && data.length === 1) {
          relationships[key] = { data: data[0] };
        } else {
          relationships[key] = { data: data };
        }
      });

      var result = {
        type: this.definition.type,
        id: id,
        attributes: attributes
      };

      // do not serialize empty object
      if (relationships && Object.keys(relationships).length > 0) {
        result.relationships = relationships;
      }

      return result;
    }
  }, {
    key: 'serializeRelationship',
    value: function serializeRelationship(relationship, relatedData, obj) {
      return {
        type: relationship.type,
        id: this.resolveIdField(relationship, relatedData, obj)
      };
    }
  }, {
    key: 'serialize',
    value: function serialize(objectOrArray) {
      var _this2 = this;

      var data = [];

      if (objectOrArray.length === undefined) {
        data.push(this.serializeObj(objectOrArray));
      } else {
        objectOrArray.forEach(function (obj) {
          data.push(_this2.serializeObj(obj));
        });
      }

      return {
        links: {},
        data: data.length === 1 ? data[0] : data
      };
    }
  }]);

  return Schema;
}();

exports.default = Schema;